' l298 driver for a picaxe08m that allows it to run 1 channel of it (so a l298 needs two picaxe8ms if you want tank drive).

' same programming for both micros!!!

' pinout is:
' pin 0 goes to a direction input on the L298
' pin 4 goes to a direction input on the L298 (the other one obviously)
' pin 2 goes to the PWM input on the L298
' pin 3 goes to read the servo pulse
' pin 1 goes to a potentiometer between +v and gnd to set whether you want freewheel or hardbrake and the movement range of the motor

' the potentiometer arc is divided in 2  halves and works like the 401 gyro. one half gives you hardbrake when the servo axis is centered
' and the other half gives you freewheel. You can move within the half to adjust the max actual PWM output to the L298 (it's symmetrical). 


' change these if you need to
symbol CENTER = 150     ' servo center in microseconds (standard is 150)
symbol DEADZONE = 10    ' servo deadzone in microseconds - prevents motor "whining"
symbol CHECKROUNDS = 10 ' how often to read pulse validity -- used for pseudo PCM checking
symbol SERVOMAX = 220   ' more than max or less than min stops the motor, decreases thrashing if using a cheap radio and radio is off
symbol SERVOMIN = 080   ' more than max or less than min stops the motor, decreases thrashing if using a cheap radio and radio is off
symbol PWMMULT = 16     ' PWM delay; should remain at 16. For the surface mount L298, use 24.

' these are only used to keep track of things, leave alone: note that freq=8mhz so periods are doubled
symbol HARDBRAKE = 200
symbol FREEWHEEL = 0
symbol C2 = CENTER + CENTER
symbol D2 = DEADZONE + DEADZONE
symbol CENTERHIGH = C2 + D2
symbol CENTERLOW  = C2 - D2
symbol DNE_FWD = SERVOMAX + SERVOMAX
symbol DNE_BAK = SERVOMIN + SERVOMIN

init:
setfreq m8
pwmout 2 off
low 0
low 4
b12 = 0

pulsechek:
pulsin 3, 1, w0
if w0 > DNE_FWD or w0 < DNE_BAK then goto init ' safety
inc b12
if b12 < CHECKROUNDS then goto pulsechek

main:
pulsin 3, 1, w0

if w0 > DNE_FWD or w0 < DNE_BAK then goto init ' safety

if w0 > CENTERHIGH then
	w0 = w0 - C2' w0 - CENTERHIGH 
	low  0
	high 4

elseif w0 < CENTERLOW then
	w0 = C2 - w0 'CENTERLOW - w0
	low  4
	high 0

else
	w0 = 2048 / PWMMULT
	high 4
	high 0
endif

w0 = w0 * PWMMULT

if w0 > 1023 then goto turnon

pwmout 2,255,w0
pause 24 ' change this if reaction time is too slow
goto main

turnon:
high 2
pause 24 ' change this if reaction time is too slow
goto main







