' l298 driver for a picaxe08m that allows it to run 1 channel of it (so a l298 needs two picaxe8ms if you want tank drive).

' pinout is:
' pin 0 goes to a direction input on the L298
' pin 1 goes to a direction input on the L298 (the other one obviously)
' pin 2 goes to the PWM input on the L298
' pin 3 goes to read the servo pulse
' pin 4 goes to a potentiometer between +v and gnd to set whether you want freewheel or hardbrake and the movement range of the motor

' the potentiometer arc is divided in 2  halves and works like the 401 gyro. one half gives you hardbrake when the servo axis is centered
' and the other half gives you freewheel. You can move within the half to adjust the max actual PWM output to the L298 (it's symmetrical). 


' change these if you need to
symbol CENTER = 150 ' servo center in microseconds
symbol DEADZONE = 3 ' servo deadzone in microseconds
symbol READTRIMPERIOD = 50 ' how often in cycles to read the trim: less often gives better response
symbol SERVOMAX = 220 ' more than max or less than min stops the motor, decreases thrashing if using a cheap radio and radio is off
symbol SERVOMIN = 080 ' more than max or less than min stops the motor, decreases thrashing if using a cheap radio and radio is off

' these are only used to keep track of things, leave alone: note that freq=8mhz so periods are doubled
symbol HARDBRAKE = 200
symbol FREEWHEEL = 0
symbol C2 = CENTER + CENTER
symbol D2 = DEADZONE + DEADZONE
symbol CENTERHIGH = C2 + D2
symbol CENTERLOW  = C2 - D2
symbol DNE_FWD = SERVOMAX + SERVOMAX
symbol DNE_BAK = SERVOMIN + SERVOMIN

init:
setfreq m8
pwmout 2 off
low 0
low 1
gosub readtrim

main:
pulsin 3, 1, w0

if w0 > DNE_FWD or w0 < DNE_BAK then goto init ' safety



if w0 > CENTERHIGH then
	w0 = w0 - CENTERHIGH
	low  0
	high 1

elseif w0 < CENTERLOW then
	w0 = CENTERLOW - w0
	low  1
	high 0

else
	w0 = w1 ' can either be 100% or 0% depending on trim
	high 1
	high 0
endif

' set pwmout
w0 = w0 * b12 max 1023
pwmout 2,255,w0

inc b11
if b11 > READTRIMPERIOD then
	gosub readtrim ' only read the trim occasionally
endif

goto main


readtrim:
readadc 4, b12
w1 = FREEWHEEL
if b12 = 0 then
	b12 = 64 ' if the pot is grounded, use  midrange
endif
if b12 > 127 then
	w1 = HARDBRAKE
	b12 = 255 - b12
endif
b12 = b12 / 8
b11 = 0
return



