' l298 driver for a picaxe08m that allows it to run 1 channel of it (so a l298 needs two picaxe8ms if you want tank drive).

' pinout is:
' pin 0 goes to a direction input on the L298
' pin 1 goes to a direction input on the L298 (the other one obviously)
' pin 2 goes to the PWM input on the L298
' pin 3 goes to read the servo pulse
' pin 4 goes to a potentiometer between +v and gnd to set whether you want freewheel or hardbrake and the movement range of the motor


' 28x1 pinout:




' change these if you need to
symbol CENTER = 150 ' servo center in microseconds
symbol DEADZONE = 3 ' servo deadzone in microseconds
symbol READTRIMPERIOD = 50 ' how +often in cycles to read the trim: less often gives better response
symbol SERVOMAX = 220 ' more than max or less than min stops the motor, decreases thrashing if using a cheap radio and radio is off
symbol SERVOMIN = 080 ' more than max or less than min stops the motor, decreases thrashing if using a cheap radio and radio is off
symbol PULSETRAINCHECK = 10 ' needs at least this many valid pulses in a row to allow for movement
symbol PWM_MULT = 12 ' step pwm output by this much

' these are only used to keep track of things, leave alone: note that freq=8mhz so periods are doubled
symbol HARDBRAKE = 200
symbol FREEWHEEL = 0
symbol C2 = CENTER + CENTER
symbol D2 = DEADZONE + DEADZONE
symbol CENTERHIGH = C2 + D2
symbol CENTERLOW  = C2 - D2
symbol DNE_FWD = SERVOMAX + SERVOMAX
symbol DNE_BAK = SERVOMIN + SERVOMIN

'#define localtelemetry  ' output nmea-like string locally  ?
#define remotetelemetry ' output nmea-like string to radio ?

symbol CH1_F = 3 ' DIRECTION pin for channel 1 (pwm is fixed)
symbol CH1_B = 2 ' DIRECTION pin for channel 1 (pwm is fixed)
symbol CH2_F = 1 ' DIRECTION pin for channel 2 (pwm is fixed)
symbol CH2_B = 0 ' DIRECTION pin for channel 2 (pwm is fixed)

symbol CH1_IN = 3 ' Servo input comes here
symbol CH2_IN = 4 ' Servo input comes here

symbol Ch1Val = w1 ' Used for both servo and PWM
symbol Ch2Val = w2 ' Used for both servo and PWM
symbol PulseCnt = b27 ' Valid pulse counter checker
symbol Adc0 = w3
symbol Adc1 = w4
symbol Adc2 = w5
symbol Adc3 = w6
symbol sonar0 =  w7
symbol sonar1 =  w8
'symbol sonar2 =  w9
'symbol sonar3 = w10


symbol RF_PIN_0 = 0
symbol RF_PIN_1 = 5
'symbol RF_PIN_2 = 3
'symbol RF_PIN_3 = 4

setfreq m8

reboot:
PulseCnt = 0

init:

pwmout 2,255,0
pwmout 1,255,0
hsersetup B19200_8, %01

low CH1_F
low CH1_B
low CH2_F
low CH2_B

pulsin CH1_IN, 1, Ch1Val
pulsin CH2_IN, 1, Ch2Val


if Ch1Val > DNE_FWD or Ch1Val < DNE_BAK or Ch2Val > DNE_FWD or Ch2Val < DNE_BAK then goto reboot
inc PulseCnt
if PulseCnt > PULSETRAINCHECK then goto main
goto init



main:


pulsin CH1_IN, 1, Ch1Val
if Ch1Val > DNE_FWD or Ch1Val < DNE_BAK then goto reboot ' safety
if Ch1Val > CENTERHIGH then
	Ch1Val = Ch1Val - CENTERHIGH
	low  CH1_F
	high CH1_B
elseif Ch1Val < CENTERLOW then
	Ch1Val = CENTERLOW - Ch1Val
	low  CH1_B
	high CH1_F
else
	Ch1Val = 0
	high CH1_F
	high CH1_B
endif
' set pwmout

Ch1Val = Ch1Val * PWM_MULT max 1023
pwmout 2,255,Ch2Val
pwmout 1,255,Ch1Val

' to speed things up, only measure half of it per cycle
readadc10 0,Adc0
readadc10 1,Adc1
gosub readleft

#ifdef remotetelemetry
hserout 0,("$PX,",#Adc0,",",#Adc1,",",#Adc2,",",#Adc3,",",#sonar0,",",#sonar1,"*",13,10)
#endif
#ifdef localtelemetry
sertxd ("$PX,",#Adc0,",",#Adc1,",",#Adc2,",",#Adc3,",",#sonar0,",",#sonar1,"*",13,10)
#endif

pulsin CH2_IN, 1, Ch2Val
if Ch2Val > DNE_FWD or Ch2Val < DNE_BAK then goto reboot ' safety
if Ch2Val > CENTERHIGH then
	Ch2Val = Ch2Val - CENTERHIGH
	low  CH2_F
	high CH2_B
elseif Ch2Val < CENTERLOW then
	Ch2Val = CENTERLOW - Ch2Val
	low  CH2_B
	high CH2_F
else
	Ch2Val = 0
	high CH2_B
	high CH2_F
endif
' set pwmout
Ch2Val = Ch2Val * PWM_MULT max 1023
pwmout 2,255,Ch2Val
pwmout 1,255,Ch1Val

' to speed things up, only measure half of it per cycle
readadc10 2,Adc2
readadc10 3,Adc3
gosub readright

#ifdef remotetelemetry
hserout 0,("$PX,",#Adc0,",",#Adc1,",",#Adc2,",",#Adc3,",",#sonar0,",",#sonar1,"*",13,10)
#endif
#ifdef localtelemetry
sertxd ("$PX,",#Adc0,",",#Adc1,",",#Adc2,",",#Adc3,",",#sonar0,",",#sonar1,"*",13,10)
#endif

goto main






readleft:
LOW PORTC RF_PIN_0
HIGH PORTC RF_PIN_0
PAUSE 1
LOW PORTC RF_PIN_0
LET dirsc = %00000110   ' puts pins back to input; also stops vehicle, for now. what's a good bitmask?
PULSIN  RF_PIN_0, 1, sonar0
return

readright:
LOW PORTC RF_PIN_1
HIGH PORTC RF_PIN_1
PAUSE 1
LOW PORTC RF_PIN_1
LET dirsc = %00000110  ' puts pins back to input; also stops vehicle, for now. what's a good bitmask?
PULSIN  RF_PIN_1, 1, sonar1
return


